//Benchmark code for a certain size. Recommended to be run with the benchmark.sh
//

#include <stdio.h>
#include <omp.h>
#include <cilk/cilk.h>
#include <cilk/cilk_api.h>
#include "efft.h"
#include <mkl_vsl.h>

int main(int argc, char* argv[]) {
  if(argc < 4) {
    printf("ERROR: Transform size, split num and number of trials must be specified.\n Exiting...\n");
    exit(1);
  }
  const long n = atol(argv[1]);
  const int numofsplits = atoi(argv[2]);
  printf("Benchmarking: Transform Size = %ld, Splits = %d, Number of Workers = %d\n", n, numofsplits, __cilkrts_get_nworkers());
  //Initializing the FFT object. This may take a few seconds as FFTW plan creation 
  // also happens on this step.
  EFFT_Transform myTransform(n, numofsplits);
  const int skipTrials = 1;
  const int nTrials = atoi(argv[3]);
  if(nTrials > skipTrials) {

    //Getting the Pointer to the data
    float* A = myTransform.Data();

    //Populating data
    const long nw=__cilkrts_get_nworkers();
    VSLStreamStatePtr rnStream[nw]; // Random number generators, one per worker
    for (int w = 0; w < nw; w++)
      vslNewStream( &rnStream[w], VSL_BRNG_MT19937, w );
    long i;
    cilk_for (i = 0; i < n; i += 1024) {
      const long w = __cilkrts_get_worker_number();
      vsRngUniform(VSL_RNG_METHOD_UNIFORM_STD, rnStream[w], 1024, &A[i], -0.5f, 0.5f);
    }

    //Benchmarking
    double gflop = 2.5*n*log2(n)/1000000000.0;
    double gflopsValues[nTrials];
    double gflopsavg = 0.0;
    double t0, t1;
    for(int i = 0; i < nTrials; i++) {
      t0 = omp_get_wtime();
      //Applies FFT to the data
      myTransform.RunTransform();
      t1 = omp_get_wtime();
      gflopsValues[i]= gflop/(t1-t0);
      gflopsavg+=(i < skipTrials) ? 0 : gflopsValues[i];
      printf("Trial %d: %f GFLOP/s\n", i , gflopsValues[i]);
    }
    gflopsavg/=nTrials-skipTrials;    
    double gflopserr = 0.0;
    for(int i = skipTrials; i < nTrials; i++) {
      gflopserr+= (gflopsValues[i]-gflopsavg) * (gflopsValues[i]-gflopsavg);
    }
    printf("Average %f GFLOP/s, Standard Deviation = %f\n", gflopsavg, sqrt(gflopserr)/(nTrials-skipTrials));

  } else {
    printf("#ERROR: Number of Trials must be at least %d\n", skipTrials+1);
  }
}
