#ifndef __INCLUDED_EFFT_H__
#define __INCLUDED_EFFT_H__

/* Enormous Fast Fourier Transform Library (EFFT) Version 1.0
   Copyright 2014, Colfax International

   Authors: ryo@colfax-intl.com     Ryo Asai
            andrey@colfax-intl.com  Andrey Vladimirov
            phi@colfax-intl.com     General inquiries

   For more information, refer to the publication
   Intel Cilk Plus for Complex Parallel Algorithms: "Enormous Fast Fourier Transforms" (EFFT) Library
   by Ryo Asai and Andrey Vladimirov (Colfax Research, 2014)
   available at http://research.colfaxinternational.com/post/2014/09/18/EFFT.aspx

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

 
#include <cilk/cilk.h>
#include <cilk/cilk_api.h>
#include <cmath>
#include <mkl.h>
#include <string>
#include <cstdio>
#include <unistd.h>

class EFFT_Transform {
  
  // Input data container
  float* dataPtr;

  // Scratch space, also used to store output
  float* scratchPtr;

  // Array of MKL DFT handles
  void* fftPlan; 

  // Transform size (N)
  const long size;

  // Number of Cooley-Tukey iterations to perform before 
  // executing single-threaded DFTs on the smaller transforms
  const long numofsplits;
  
public:

  // Constructor: N becomes "size", splits becomes "numofsplits"
  EFFT_Transform(const long N, const long splits);

  // Destructor: free the dataPtr and scratchPth
  ~EFFT_Transform();
  
  // Run the DFT on the data in dataPtr and put the result in scratchPtr
  void RunTransform();
    
  // Accessor to input data
  float* Data() { return dataPtr; }

  // Accessor to output data
  float* Result() { return scratchPtr; }
};

#endif
